@@ -1,7 +1,7 @@
 from flask_admin.contrib import sqla, fileadmin
 from flask.ext.admin.base import expose
 
-from shelf.admin.form.fields import LocalizedTextField
+from shelf.admin.form.fields import LocalizedTextField, WysiwygTextField, LocalizedWysiwygTextField
 from shelf.model.base import shelf_computed_models
 
 import os
@@ -9,23 +9,39 @@ import os.path as op
 from operator import itemgetter
 
 from flask import request
+from sqlalchemy.types import Text
 
 from base64 import b64decode
 
+from jinja2 import contextfunction
+
 class ShelfModelView(sqla.ModelView):
     list_template = "shelf-admin/model/list.html"
     create_template = "shelf-admin/model/create.html"
     edit_template = "shelf-admin/model/edit.html"
 
+    @contextfunction
+    def get_list_value(self, context, model, name):
+        if isinstance(getattr(model, name), shelf_computed_models()['LocalizedString']) \
+              or isinstance(getattr(model, name), shelf_computed_models()['LocalizedText']):
+            return getattr(model, name).value
+        return super(ShelfModelView, self).get_list_value(context, model, name)
+
     def scaffold_form(self):
         cls = super(ShelfModelView, self).scaffold_form()
         #print self.model
         for k, v in self.model.__dict__.items():
-            try:
-                if issubclass(v.mapper.class_, shelf_computed_models()['LocalizedString']):
-                    setattr(cls, k, LocalizedTextField())
-            except:
-                pass
+            if hasattr(v, "type") and isinstance(v.type, Text):
+                setattr(cls, k, WysiwygTextField())
+
+            if hasattr(v, "mapper"):
+                if issubclass(v.mapper.class_, shelf_computed_models()['LocalizedString']) \
+                    and not issubclass(self.model, shelf_computed_models()['LocalizedString']):
+                        setattr(cls, k, LocalizedTextField())
+                elif issubclass(v.mapper.class_, shelf_computed_models()['LocalizedText']) \
+                    and not issubclass(self.model, shelf_computed_models()['LocalizedText']):
+                        setattr(cls, k, LocalizedWysiwygTextField())
+
         return cls
 
 
